﻿using UnityEngine;

namespace Magify
{
    public class MagifyServiceArgs
    {
#if !MAGIFY_3_OR_NEWER
        public MagifyEditorMoq EditorMoq { get; init; }
#endif
        public IPopupsProvider PopupsProvider { get; init; }

        /// <summary>
        /// Default logging status. You can change it later using <see cref="MagifyService.IsLoggingEnabled"/>
        /// </summary>
        public bool IsLoggingEnabled { get; set; } = true;

        /// <summary>
        /// Default environment. You can change it later using <see cref="MagifyService.Environment"/>
        /// </summary>
        public Environment Environment { get; set; } = Environment.Staging;

        /// <summary>
        /// Sets the source config for editor mode.
        /// </summary>
        /// <example>
        /// <code>
        /// RuntimePlatform.IPhonePlayer -> MagifySettings.ConfigIOS
        /// RuntimePlatform.Android and other -> MagifySettings.ConfigGP
        /// </code>
        /// </example>
        public RuntimePlatform EditorConfigMode { get; set; } = RuntimePlatform.Android;

        /// <summary>
        /// State of user consent to GDPR
        /// </summary>
        /// <remarks>If the value is null, default value(false) or saved value from previous session will be used.</remarks>
        public bool? IsGdprApplied { get; set; }

        /// <inheritdoc cref="MagifyConfig.IsDeveloperMode"/>
        public bool IsDeveloperMode { get; set; }
    }
}