using System.Collections.Generic;
using JetBrains.Annotations;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;

namespace Magify
{
    internal static class JsonFacade
    {
        public static JsonSerializerSettings DefaultSettings { get; } = new()
        {
            NullValueHandling = NullValueHandling.Ignore,
            Converters = new List<JsonConverter>
            {
                new DictionaryConverter(),
                new StringNullableEnumConverter(),
                new StringEnumConverter()
            }
        };

        public static string SerializeObject(object value)
        {
            return JsonConvert.SerializeObject(value, DefaultSettings);
        }

        public static string SerializeObject(object value, JsonSerializerSettings settings)
        {
            return JsonConvert.SerializeObject(value, settings);
        }

        public static string SerializeObject(object value, Formatting formatting)
        {
            return JsonConvert.SerializeObject(value, formatting, DefaultSettings);
        }

        [CanBeNull]
        public static T DeserializeObject<T>(string json)
        {
            return JsonConvert.DeserializeObject<T>(json, DefaultSettings);
        }

        public static bool IsNullOrEmpty(string json)
        {
            return string.IsNullOrEmpty(json) || json == "null";
        }
    }
}